#----------------------------------------------------------------------------
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/system/Sys.pm,v 1.49 2014/02/20 12:32:36 kkorde Exp $
#
# Veritas Software Corporation
#
# $Author: kkorde $
# $Date: 2014/02/20 12:32:36 $
# $Revision: 1.49 $
#
# Provides: package Sys
#
# Functionality: wrapper to 'system' calls
#                as well as OS calls ping, mail, reboot
#
# Authors: 011787
#----------------------------------------------------------------------------

#----------------------
# version calls.
#----------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/system/Sys.pm,v 1.49 2014/02/20 12:32:36 kkorde Exp $::ETVHeader");
#-------------------------
# end of version calls.
#-------------------------

#---------------------------------
# Declare the package name.
#---------------------------------
package Sys;

#-------------------------------------
# Be strict on subs, vars and refs.
#-------------------------------------
use strict;

#-------------------------------------
# Include all the modules needed.
#-------------------------------------
use Net::Ping;
use LogInt;
use File::stat;
use File::Path qw ( mkpath );
use POSIX qw ( setuid );

#--------------------------
# define system commands.
#--------------------------
$::gsCommandSu          = "/bin/su";
$::gsCommandArgsSu      = "-c";
$::gsCommandReboot      = "/usr/sbin/reboot";
$::gsCommandArgsReboot  = "-q";
$::gsCommandPing        = "/usr/sbin/ping";
$::giPingTimeoutSeconds = 5;
$::gsCommandSendmail    = "/usr/lib/sendmail -t";


#----------------------------------------------
# Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
#----------------------------------------------
$::gsCommandHasys = ( $ENV { CLUSTER_HOME } ?
                      $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) . "/bin/hasys";

=pod

----------------------------------------------------------------------------
Function: Boolean RunCmdArgs( IN String Command,
                              IN String CommandArgs,
                              OUT Integer CommandReturnCode )

Description:
  Executes the specified command with given arguments. The exit code by
  the command is returned to the caller.

Parameters:
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments to the command.
  CommandReturnCode : the exit code by the command executed.

Return Value:
  Boolean value from the set of {0,1} where
         0 - fialed
         1 - success
----------------------------------------------------------------------------

=cut

sub RunCmdArgs {

  my ( $sCommand,
       $sCommandArgs,
       $riCommandReturnCode ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10140,
  "Sys",
  "RunCmdArgs",
  "Subroutine <%s> called with:
  Command     [%s]
  Arguments   [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  if(-x $sCommand) {
    LogInt::Message( LogInt::LLTrace,
                     10141,
                     "Sys",
                     "RunCmdArgs",
                     "Going to run executable file at [%s], with args [%s]",
                     $sCommand,
                     $sCommandArgs );

    my $sCommandLine = "$sCommand $sCommandArgs";

    #------------------
    # run the command.
    #------------------
    system("$sCommandLine");
    $$riCommandReturnCode = $? >> 8;
    LogInt::Message( LogInt::LLTrace,
                     10142,
                     "Sys",
                     "RunCmdArgs",
                     "Command [%s] args [%s] returned code [%s]",
                     $sCommand,
                     $sCommandArgs,
                     $$riCommandReturnCode );

    return 1; #return good
  }
  else {
    LogInt::Message( LogInt::LLError,
                     10143,
                     "Sys",
                     "RunCmdArgs",
                     "Failed to find an executable file at [%s]",
                     $sCommand );

    return 0; #return error
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean RunCmdArgsWithTimeout( IN String Command, 
                                         IN String CommandArgs, 
                                         OUT Integer CommandReturnCode,
                                         IN Integer TimeoutSeconds, 
                                         OUT Boolean TimedOut )

Description:
  Executes the given command with given arguments with specified time.

Parameters:
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments to the command.
  CommandReturnCode : the exit code by the command executed.
  TimeoutSeconds    : Number of seconds within which the command 
                      expected to executed.
  TimedOut          : Return to the caller whether the command is timed out 
                      or executed successfully.
 
Return Value:
  Boolean value from the set of {0,1} where
         0 - failed
         1 - success
----------------------------------------------------------------------------

=cut

sub RunCmdArgsWithTimeout {

  my ( $sCommand,
       $sCommandArgs, 
       $riCommandReturnCode,
       $iTimeoutSeconds,
       $rbTimedOut ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message ( LogInt::LLTrace,
  10144,
  "Sys",
  "RunCmdArgsWithTimeout",
  "Subroutine <%s> called with:
  Command         [%s]
  Arguments       [%s]
  Timeout         [%s]", 
  $sMyName, $sCommand, $sCommandArgs, $iTimeoutSeconds );

  #-----------------------------------------------------------
  # call the normal runcmd but with timeout.  
  # set up 'try' block for the SIGALARM so we can timeout 
  # after iTimeoutSeconds
  #-----------------------------------------------------------
  $$rbTimedOut = 1; #set true to start with
  my $iRunCmdArgsReturnCode = 0;

  eval {
    local $SIG{ALRM} = sub {die "sigalarm jump eval\n"};
    alarm $iTimeoutSeconds;

    $iRunCmdArgsReturnCode = RunCmdArgs($sCommand, $sCommandArgs, $riCommandReturnCode);

    alarm 0; #cancel the alarm
    $$rbTimedOut = 0; #command didnt timeout

    LogInt::Message( LogInt::LLTrace,
                     10145,
                     "Sys",
                     "RunCmdArgsWithTimeout",
                     "Timed command [%s] completed within period", 
                     $sCommand );
  };

  return $iRunCmdArgsReturnCode;
}


=pod

----------------------------------------------------------------------------
Function: Boolean RunCmdArgsAsUser( IN String Command, 
                                    IN String CommandArgs, 
                                    OUT Integer CommandReturnCode, 
                                    IN String UserId)

Description:
  Executes the command with the specified user.
	
Parameters:
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments to the command.
  CommandReturnCode : the exit code by the command executed.
  UserId            : User ID with which the command needs to be executed.

Return Value:
  Boolean value in the set of {0,1} where
         0 - failed
         1 - success 
----------------------------------------------------------------------------

=cut

sub RunCmdArgsAsUser {

  my ( $sCommand,
       $sCommandArgs,
       $riCommandReturnCode, 
       $sUserId ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10146,
  "Sys",
  "RunCmdArgsAsUser",
  "Subroutine <%s> called with:
  Command       [%s]
  Arguments     [%s]
  User          [%s]",
  $sMyName, $sCommand, $sCommandArgs, $sUserId );

  if(-x $sCommand) {
    LogInt::Message( LogInt::LLTrace,
                     10147,
                     "Sys",
                     "RunCmdArgsAsUser",
                     "Going to run executable file at [%s] args [%s] user [%s]",
                     $sCommand, 
                     $sCommandArgs,
                     $sUserId );

    my $sCommandLine = "$::gsCommandSu $sUserId $::gsCommandArgsSu '$sCommand $sCommandArgs'";

    LogInt::Message( LogInt::LLTrace,
                     10148,
                     "Sys",
                     "RunCmdArgsAsUser",
                     "Command line [%s]",
                     $sCommandLine );

    #-------------------
    # run the command.
    #-------------------
    system("$sCommandLine");

    $$riCommandReturnCode = $? >> 8;

    LogInt::Message( LogInt::LLTrace, 
                     10149,
                     "Sys",
                     "RunCmdArgsAsUser",
                     "Command [%s] args [%s] returned code [%s]",
                     $sCommand,
                     $sCommandArgs,
                     $$riCommandReturnCode );

    return 1; #return good
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10150,
                     "Sys",
                     "RunCmdArgsAsUser",
                     "Failed to find an executable file at [%s]",
                     $sCommand );

    return 0; #return error
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean RunCmdArgsAsUserWithTimeout( IN  String Command, 
            	                               IN  String CommandArgs, 
                        	               OUT Integer CommandReturnCode, 
                                    	       IN  String UserId, 
                                               IN  Integer TimeoutSeconds, 
                                               OUT Integer TimedOut )

Description:
  Executes the command with the user and timeout.

Parameters:
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments to the command.
  CommandReturnCode : the exit code by the command executed.
  UserId            : User ID with which the command needs to be executed.
  TimeoutSeconds    : Number of seconds within which the command 
                      expected to executed.
  TimedOut          : Return to the caller whether the command is 
                      timed out or executed successfully.

Return Value:
  Boolean value in the set of {0,1} where
         0 - failed
         1 - success 
----------------------------------------------------------------------------

=cut

sub RunCmdArgsAsUserWithTimeout {

  my ( $sCommand,
       $sCommandArgs,
       $riCommandReturnCode,
       $sUserId,
       $iTimeoutSeconds, 
       $rbTimedOut ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10151,
  "Sys",
  "RunCmdArgsAsUserWithTimeout",
  "Subroutine <%s> called with:
  Command         [%s]
  Arguments       [%s]
  User            [%s]
  Timeout         [%s]", 
  $sMyName, $sCommand, $sCommandArgs, $sUserId, $iTimeoutSeconds );

  #--------------------------------------------------
  # call the normal runcmd as user but with timeout.
  # set up 'try' block for the SIGALARM so we can 
  # timeout after iTimeoutSeconds.
  #--------------------------------------------------
  $$rbTimedOut = 1; #set true to start with
  my $iRunCmdArgsAsUserReturnCode = 0;

  eval {
    local $SIG{ALRM} = sub {die "sigalarm jump eval\n"};
    alarm $iTimeoutSeconds;

    $iRunCmdArgsAsUserReturnCode = RunCmdArgsAsUser($sCommand, $sCommandArgs, $riCommandReturnCode, $sUserId);

    alarm 0; #cancel the alarm
    $$rbTimedOut = 0; #command didnt timeout

    LogInt::Message( LogInt::LLTrace,
                     10152, 
                     "Sys",
                     "RunCmdArgsAsUserWithTimeout",
                     "Timed command [%s] completed within period", 
                     $sCommand );
  };

  return $iRunCmdArgsAsUserReturnCode;
}


=pod

----------------------------------------------------------------------------
Function: void QuickSystemShutdown( IN String Message )
	
Description:
  Shutdowns the system.

Parameters:
  Message : Message to print while shutting down.

Return Value:
  void : No value is returned the caller.
----------------------------------------------------------------------------

=cut

sub QuickSystemShutdown {

  my ( $sMessage ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10153,
  "Sys",
  "QuickSystemShutdown",
  "Subroutine <%s> called with:
  Message        [%s]",
  $sMyName, $sMessage );

  LogInt::Message( LogInt::LLWarn, 
                   10154,
                   "Sys", 
                   "QuickSystemShutdown",
                   "Shutting Down calling [%s] [%s]",
                   $::gsCommandReboot,
                   $::gsCommandArgsReboot );

  my $iRebootReturnCode;
  if(RunCmdArgs($::gsCommandReboot, $::gsCommandArgsReboot, \$iRebootReturnCode)) {
    LogInt::Message( LogInt::LLWarn,
                     10155,
                     "Sys",
                     "QuickSystemShutdown",
                     "RunCmdArgs exited" );
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10156,
                     "Sys",
                     "QuickSystemShutdown",
                     "RunCmdArgs failed" );
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean Ping( IN String Host )

Description:
  Pings the host machine and checks it is reachable or not.

Parameters:
  Host : Name of the host to ping.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
----------------------------------------------------------------------------

=cut

sub Ping {

  my ( $sHost ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10157,
  "Sys",
  "Ping",
  "Subroutine <%s> called with:
  Host     [%s]",
  $sMyName, $sHost );

  LogInt::Message( LogInt::LLTrace, 
                   10158,
                   "Sys",
                   "Ping",
                   "Pinging host [%s] with timeout [%s] seconds",
                   $sHost,
                   $::giPingTimeoutSeconds );

  #-----------------------------
  # Initialize the ping object.
  #-----------------------------
  my $p = Net::Ping->new("icmp", $::giPingTimeoutSeconds );

  #--------------------------------------------
  # 'loose' any stdout or stderr from ping.
  #--------------------------------------------
  # my $sPingArgs = sprintf "%s %d > /dev/null 2>&1", $sHost, $::giPingTimeoutSeconds;

  my $iPingReturnCode;
  if( $p->ping($sHost) ) {
    #-----------------
    # pinged OK.
    #-----------------
    $p->close();
    LogInt::Message( LogInt::LLTrace,
                     10159,
                     "Sys",
                     "Ping",
                     "Pinging host [%s] with timeout [%s] seconds (Ping OK)",
                     $sHost,
                     $::giPingTimeoutSeconds );
    return 1;
  } 
  else {
    #--------------------
    # failed to ping.
    #--------------------
    $p->close();
    LogInt::Message( LogInt::LLTrace,
                     10160,
                     "Sys",
                     "Ping",
                     "Pinging host [%s] with timeout [%s] seconds (Failed to Ping)",
                     $sHost,
                     $::giPingTimeoutSeconds );
    return 0;
  }
}


=pod

----------------------------------------------------------------------------
Function: void SendMail( IN String Recipient, 
                         IN String Subject, 
                         IN String Message )

Description:
  Sends a mail to the specified recipient.

Parameters:
  Recipient : Address of the recipient.
  Subject   : Subject for the message/mail.
  Message   : Actual message to send.

Return Value:
  void - No value is returned to the caller.
----------------------------------------------------------------------------

=cut

sub SendMail {

  my ( $sRecipient,
       $sSubject,
       $sMessage ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10161,
  "Sys",
  "SendMail",
  "Subroutine <%s> called with:
  Recipient        [%s]
  Subject          [%s]", 
  $sMyName, $sRecipient, $sSubject );

  LogInt::Message( LogInt::LLTraceTrace,
                   10162, 
                   "Sys", 
                   "SendMail",
                   "Message body [%s]", $sMessage );

  my $iChildPid = open( MFILE, "| $::gsCommandSendmail")
             or LogInt::Message( LogInt::LLError,
                                 10163, 
                                 "Sys", 
                                 "SendMail",
                                 "Can't fork a process for sendmail: %s",
                                 $! );
  if( defined $iChildPid && $iChildPid > 1 ) {
    print MFILE "To: $sRecipient\nFrom: SendMail\nSubject: $sSubject\n$sMessage\n\n";
  }
  else {
    LogInt::Message( LogInt::LLError,
                     10164,
                     "Sys",
                     "SendMail",
                     "The attempt to open a pipe to command [%s] resulted in an invalid child PID of [%s]",
                     $::gsCommandSendmail,
                     $iChildPid );
  }

  my $sLastOS_Error = $!;
  my $iExitCode = undef;
  if( close(CMD) ) {
    $iExitCode = $? >> 8;
  }
  else {
    $iExitCode = $? >> 8;
    #-----------------------------------------------------------------------
    # Commands that exit non-zero, which sometimes does not indicate that
    # it failed, causes the close to return zero instead of >= 1.
    #-----------------------------------------------------------------------
    LogInt::Message( LogInt::LLTrace,
                     10165,
                     "Sys",
                     "SendMail",
                     "Command line [%s] provided a non-zero exit code[%s] -- this does not necessarily indicate a problem... (Perl's OS error variable prior to the command-pipe close was [%s], and after the close was [%s] )",
                     $::gsCommandSendmail, $iExitCode, $sLastOS_Error, $! );
  }

  LogInt::Message( LogInt::LLTrace, 
                   10166, 
		   "Sys",
		   "SendMail",
		   "Mail sent" );
  
  return;
}


=pod

----------------------------------------------------------------------------
Function: Boolean DeleteFile( IN String FileName )

Description:
  Deletes the file specified.

Parameters:
  FileName : Name of the file to be deleted.

Return Value:
  Boolean value in the set of {0,1} where
        0 - failed
        1 - success 
----------------------------------------------------------------------------

=cut

sub DeleteFile {

  my ( $sFileName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10167,
  "Sys",
  "DeleteFile",
  "Subroutine <%s> called with:
  Filename      [%s]",
  $sMyName, $sFileName );

  if(-e $sFileName) {
    #-------------------------
    # file exists - continue.
    #-------------------------
  } 
  else {
    #-------------------------
    # treat this as 'sucess'.
    #-------------------------
    LogInt::Message( LogInt::LLTrace,
                     10168,
                     "Sys",
                     "DeleteFile",
                     "File [%s] did not exist", 
                     $sFileName );
    return 1; #return ok
  }

  my $iUnlinkReturnCode = unlink($sFileName);

  if($iUnlinkReturnCode == 1) {
    LogInt::Message( LogInt::LLTrace,
                     10169,
                     "Sys",
                     "DeleteFile",
                     "Delete file [%s] unlink status OK", 
                     $sFileName );
    return 1; #rm cmd worked ok
  } 
  else {
    LogInt::Message( LogInt::LLWarn,
                     10170, 
                     "Sys",
                     "DeleteFile",
                     "Delete file [%s] failed to unlink",
                     $sFileName );
    return 0; #error occured
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean FileExists( IN String File )

Description: 
  Checks status of a given file.

Parameters:
  File : File name which needs to check status.

Return Value:
  Boolean value from the set of {0,1} where
         0 - failed
         1 - success
----------------------------------------------------------------------------

=cut

sub FileExists {

  my ( $sFileName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10171,
  "Sys",
  "FileExists",
  "Subroutine <%s> called with:
  Filename     [%s]", 
  $sMyName, $sFileName );

  if(-e $sFileName) {
    LogInt::Message( LogInt::LLTrace,
                     10172,
                     "Sys",
                     "FileExists",
                     "File [%s] exists",
                     $sFileName );
    return 1; #return true
  } 
  else {
    LogInt::Message( LogInt::LLTrace,
                     10173,
                     "Sys",
                     "FileExists",
                     "File [%s] does not exist",
                     $sFileName );
    return 0; #return false
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean GetSystemNameAsVcsKnowsIt( OUT String SystemName )

Description:
  Gets the system name using hasys command.

Parameters:
  SystemName : The system name given by hasys command.

Return Value:
  Boolean value from the set of {0,1} where
          0 - failed
          1 - success
----------------------------------------------------------------------------

=cut

sub GetSystemNameAsVcsKnowsIt {

  my ( $rsCurrentSystemName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10174,
  "Sys", 
  "GetSystemNameAsVcsKnowsIt",
  "Subroutine <%s> called", $sMyName );

  #--------------------
  # call the command.
  #--------------------
  my $sHasysOut = `$::gsCommandHasys -nodeid 2>&1`;

  my $iReturnCode = $? >> 8;

  #-----------------------------------------------------------
  # check the return code if not 0 then print an error message
  # using the returned data.
  #-----------------------------------------------------------
  if ($iReturnCode != 0) {
    LogInt::Message( LogInt::LLError, 
                     10175,
                     "Sys",
                     "GetSystemNameAsVcsKnowsIt",
                     "Failed with message [%s]",
                     $sHasysOut );
    return 0; #error return
  } 
  else {
    #--------------------------------------------------------------------
    # command worked
    # format should be 
    # TAG_D 2001/06/04 20:45:47 VCS:10753:Node name: hydrogen Node ID: 1
    #--------------------------------------------------------------------

    chomp $sHasysOut;

    $sHasysOut =~ s/.*Node name: //;
    $sHasysOut =~ s/ Node ID: .*//;

    $$rsCurrentSystemName = $sHasysOut;

    LogInt::Message( LogInt::LLTrace,
                     10176,
                     "Sys",
                     "GetSystemNameAsVcsKnowsIt",
                     "Got current system name [%s]",
                     $$rsCurrentSystemName );

    return 1; #good status return
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean RunWithEnvCmdWithOutput( IN String EnvFile,
                                           IN Boolean SourceEnv,
                                           IN String Command,
                                           IN String CommandArgs,
                                           OUT Integer CommandReturnCode ,
                                           IN String User,
                                           OUT Array CommandOut ) 

Description:
  This function executes the specified command by sourcing environment file 
  if needed and with specified user.

Parameters:
  EnvFile           : Environment file that needs to be sourced before executing the command. 
  SourecEnv         : A flag to indicate whether the environment file needs to be sourced or not.
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments that needs to be passed to the command.
  CommandReturnCode : The return code of the command executed.
  User              : The user name with which the command needs to be executed.
  CommandOut        : The output of the command executed.

Return Value:
  Boolean value from the set of {0,1} where
         0 - failed
         1 - success
----------------------------------------------------------------------------

=cut

sub RunWithEnvCmdWithOutput {

  my ( $sEnvFile,
       $bSourceEnv,
       $sCommand,
       $sCommandArgs,
       $riCommandReturnCode,
       $sUser,
       $rlsCommandOut ) = @_ ;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10177,
  "Sys",
  "RunWithEnvCmdWithOutput",
  "Subroutine <%s> called with:
  EnvFile           [%s]
  Command           [%s]
  Arguments         [%s]
  User              [%s]",
  $sMyName, $sEnvFile, $sCommand, $sCommandArgs, $sUser );

  #----------------------------
  # Establish local variables.
  #----------------------------
  my $sCmdToRun    ;
  my $sCommandLine ;

  #--------------------------------
  # Confirm existance of EnvFile.
  #--------------------------------
  if ( $sEnvFile eq "" ) {
    LogInt::Message( LogInt::LLTrace,
                     10178,
                     "Sys",
                     "RunWithEnvCmdWithOutput",
                     "Environment file not set" );
  }
  elsif ( ! -e $sEnvFile ) {
    LogInt::Message( LogInt::LLError,
                     10179,
                     "Sys",
                     "RunWithEnvCmdWithOutput",
                     "Environment file [%s] does not exist",
                     $sEnvFile );
    return 0 ;
  } 
  else {
    my ( $sName      ,
         $sPassword  ,
         $iUid       ,
         $iGid       ,
         $iQuota     ,
         $sComment   ,
         $sGcos      ,
         $sHome      ,
         $sUserShell ,
         $sExpire    ) = getpwnam($sUser);

    if ( $sUserShell =~ /(.*)\/bin\/.*csh$/ ) {
      LogInt::Message( LogInt::LLTrace,
                       10180,
                       "Sys",
                       "RunWithEnvCmdWithOutput"   ,
                       "User [%s] has csh derived native shell [%s]",
                       $sUser,
                       $sUserShell );
      #---------------------------------------
      # Shell is a csh varient.  Use source.
      #---------------------------------------
      $sCmdToRun = "setenv USER $sUser; setenv HOME $sHome; source $sEnvFile;";
    } 
    else {
      LogInt::Message( LogInt::LLTrace,
                       10181,
                       "Sys",
                       "RunWithEnvCmdWithOutput"   ,
                       "User [%s] has sh derived native shell [%s]",
                       $sUser,
                       $sUserShell );
      #--------------------------------------------------------------------------
      # Shell is one of the non-C type shells. Use .  to source in the env file.
      #--------------------------------------------------------------------------
      $sCmdToRun = "USER=$sUser; HOME=$sHome; export USER HOME; . $sEnvFile ;";
    }
  }

  #-------------------------------------
  # Confirm existance of Exectuable.
  #-------------------------------------
  if ( ! -e $sCommand ) {
    LogInt::Message( LogInt::LLError,
                     10182, 
                     "Sys",
                     "RunWithEnvCmdWithOutput",
                     "File [%s] does not exist", 
                     $sCommand ) ;
    return 0 ;
  }
  elsif ( ! -x $sCommand ) {
    LogInt::Message( LogInt::LLError,
                     10183,
                     "Sys",
                     "RunWithEnvCmdWithOutput",
                     "File [%s] is not executable",
                     $sCommand ) ;
    return 0 ;
  }
  else {
    if( defined $sCommandArgs ) {
      $sCmdToRun .= "$sCommand  $sCommandArgs" ;
    }
    else {
      $sCmdToRun .= $sCommand;
    }
  }
   
  LogInt::Message( LogInt::LLTrace,
                   10184,
                   "Sys", 
                   "RunWithEnvCmdWithOutput", 
                   "Going to run [%s], with ENV [%s], with args [%s] as user [%s]", 
                   $sCommand,
                   $sEnvFile,
                   $sCommandArgs,
                   $sUser );

  if ( $bSourceEnv ) {
    LogInt::Message( LogInt::LLTrace,
                     10185,
                     "Sys", 
                     "RunWithEnvCmdWithOutput",
                     "Sourcing Environment of [%s]", 
                     $sUser );
    $sCommandLine = "$::gsCommandSu - $sUser $::gsCommandArgsSu '$sCmdToRun'";
  } 
  else {
    $sCommandLine = "$::gsCommandSu $sUser $::gsCommandArgsSu '$sCmdToRun'";
  }

  LogInt::Message( LogInt::LLTraceTrace,
                   10186,
                   "Sys",
                   "RunWithEnvCmdWithOutput",
                   "Going to run [%s]",
                   $sCommandLine );

  @$rlsCommandOut = `$sCommandLine 2>&1`;
  $$riCommandReturnCode = $? >> 8;

  foreach my $sCommandLineOut (@$rlsCommandOut) {
    chomp($sCommandLineOut);
    LogInt::Message( LogInt::LLTrace,
                     10187, 
                     "Sys",
                     "RunWithEnvCmdWithOutput",
                     "Command OUTPUT: [%s]", 
                     $sCommandLineOut );
  }

  LogInt::Message( LogInt::LLTrace,
                   10188,
                   "Sys",
                   "RunWithEnvCmdWithOutput",
                   "Command [%s] with args [%s] returned code [%s]",
                   $sCommand,
                   $sCommandArgs,
                   $$riCommandReturnCode );
  return 1;
}


=pod

----------------------------------------------------------------------------
Function: Boolean RunWithEnvCmdWithOutputWithTimeOut( 
                                     IN  String    EnvFile,
                                     IN  Boolean   SourceEnv,
                                     IN  String    Command,
                                     IN  String    CommandArgs,
                                     OUT Integer   CommandReturnCode ,
                                     IN  String    User,
                                     OUT Array     CommandOut ,
                                     OUT Boolean   TimedOut,
                                     IN  Integer   TimeoutSeconds ,
                                     IN  String    OutFileName ,
                                     IN  String    FromDir );

Description:
  This function executes the specified command by sourcing environment file
  if needed and with specified user. It executes the command with timeout value.

Parameters:
  EnvFile           : Environment file that needs to be sourced before 
                      executing the command. 
  SourecEnv         : A flag to indicate whether the environment file 
                      needs to be sourced or not.
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments that needs to be passed to the command.
  CommandReturnCode : The return code of the command executed.
  User              : The user name with which the command needs to be executed.
  CommandOut        : The output of the command executed.
  TimeOut           : Flag indicating whether the command failed to execute 
                      due to timeout.
  TimeoutSeconds    : Maximum time the command can execute before getting
                      signaled out due to timeout.
  OutFileName       : Output of the command will be redirected to this file,
                      only if specified. NULL value is permitted.
  FromDir           : Command will be executed after doing a chdir to this dir.
  CryptString       : String to hide in CommandArgs

Return Value:
  Boolean value from the set of {0,1} where
         0 - command timedout
         1 - command not timedout.
----------------------------------------------------------------------------

=cut

sub RunWithEnvCmdWithOutputWithTimeOut {

  my ( $sEnvFile,
       $bSourceEnv,
       $sCommand,
       $sCommandArgs,
       $riCommandReturnCode,
       $sUser,
       $rlsCommandOut,
       $rbTimedOut,
       $iTimeoutSeconds,
       $sOutFileName,
       $sFromDir,
       $sCryptString ) = @_ ;

  my $sMyName = ( caller 0 )[3];
  my $sCrArgs = $sCommandArgs; # Backup..
  my $sDispCmd = undef;

  #----------------------------------
  # Hide from display, if requested..
  #----------------------------------
  if ( defined $sCryptString ) {
    $sCryptString =~ s/([^\w-])/./g;
    $sCrArgs =~ s/$sCryptString/XXXXXXXXXX/g;
  }

  LogInt::Message ( LogInt::LLTrace,
  10189,
  "Sys",
  "RunWithEnvCmdWithOutputWithTimeOut",
  "Subroutine <%s> called with:
  EnvFile       [%s]    
  Command       [%s]
  Arguments     [%s]
  User          [%s]
  Timeout       [%s]
  OutFile       [%s]
  FromDir       [%s]",
  $sMyName, $sEnvFile, $sCommand, $sCrArgs, $sUser,
  $iTimeoutSeconds, $sOutFileName, $sFromDir );

  #-----------------------------
  # Establish local variables.
  #-----------------------------
  my $sCmdToRun     = "";
  my $sCommandLine  = "";

  if ( ! defined $sUser || $sUser eq "" ) {
    LogInt::Message( LogInt::LLTrace,
                     10190,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "No user is set. Setting User to root." );

    $sUser = "root";
  }

  my ( $sName      ,
       $sPassword  ,
       $iUid       ,
       $iGid       ,
       $iQuota     ,
       $sComment   ,
       $sGcos      ,
       $sHome      ,
       $sUserShell ,
       $sExpire    ) = getpwnam($sUser);

  my $bCshSet       = 0;
  if ( $sUserShell =~ /(.*)\/bin\/.*csh$/ )
  {
    $bCshSet = 1;
  }

  #--------------------------------
  # Confirm existance of EnvFile.
  #--------------------------------
  if ( $sEnvFile eq "" ) {
    LogInt::Message( LogInt::LLTrace,
                     10191,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "Environment file not set" );
  } 
  elsif ( ! -e $sEnvFile ) {
    LogInt::Message( LogInt::LLError,
                     10192,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "Environment file [%s] does not exist",
                     $sEnvFile );
    return 0 ;
  } 
  else {

    if ( $sUserShell =~ /(.*)\/bin\/.*csh$/ ) {
      LogInt::Message( LogInt::LLTrace,
                       10193,
                       "Sys",
                       "RunWithEnvCmdWithOutputWithTimeOut",
                       "User [%s] has csh derived native shell [%s]",
                       $sUser,
                       $sUserShell );
      #---------------------------------------
      # Shell is a csh varient.  Use source.
      #---------------------------------------
      if( $::giLogIntLogLevel == LogInt::LLTraceTrace ) {
        $sCmdToRun = "setenv USER $sUser; setenv HOME $sHome; source $sEnvFile; /bin/env;";
      }
      else {
        $sCmdToRun = "setenv USER $sUser; setenv HOME $sHome; source $sEnvFile;";
      }
    }
    else {
      LogInt::Message( LogInt::LLTrace,
                       10194,
                       "Sys",
                       "RunWithEnvCmdWithOutputWithTimeOut",
                       "User [%s] has sh derived native shell [%s]",
                       $sUser, 
                       $sUserShell );
      #----------------------------------------------
      # Shell is one of the non-C type shells. Use .
      # to source in the env file.
      #----------------------------------------------
      if( $::giLogIntLogLevel == LogInt::LLTraceTrace ) {
        $sCmdToRun = "USER=$sUser; HOME=$sHome; export USER HOME ; . $sEnvFile ; /bin/env;";
      }
      else {
        $sCmdToRun = "USER=$sUser; HOME=$sHome; export USER HOME ; . $sEnvFile ;";
      }
    }
  }

  #---------------------------------------------
  # Run from specified directory.
  #---------------------------------------------
  $sFromDir ||= ''; # Prevent undefs..

  if ( $sFromDir =~ /\S/ ) { # Now can be called with ""..
    if ( -d $sFromDir ) {
      $sCmdToRun .= "cd $sFromDir ;";
    }
    else {
      LogInt::Message( LogInt::LLError,
                     10195,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "Directory [%s] does not exist",
                     $sFromDir ) ;
      return 0;
    }
  }

  #------------------------------------
  # Confirm existance of Exectuable.
  #------------------------------------
  if ( ! -e $sCommand ) {
    LogInt::Message( LogInt::LLError,
                     10196,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "File [%s] does not exist",
                     $sCommand ) ;
    return 0 ;
  } 
  elsif ( ! -x $sCommand ) {
    LogInt::Message( LogInt::LLError,
                     10197,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "File [%s] is not executable", 
                     $sCommand ) ;
    return 0 ;
  } 
  else {
    if( defined $sCommandArgs || $sCommandArgs ne "" ) {
      $sCmdToRun .= "$sCommand $sCommandArgs" ;
    }
    else {
      $sCmdToRun .= $sCommand ;
    }
  }

  $sOutFileName ||= ""; # Prevent undefs..

  if ( $sOutFileName =~ /\S/ ) { # Now allow calling with ""..
    #-----------------------------
    # Create basedir for outfile..
    #-----------------------------
    ( my $sBaseDir = $sOutFileName ) =~ s#(.*)/.*$#$1#;
    unless ( -d $sBaseDir ) {
      if ( ! mkpath ( $sBaseDir, 0, 0755 ) ) {
        LogInt::Message( LogInt::LLError,
        10198,
        "Sys",
        "RunWithEnvCmdWithOutputWithTimeOut",
        "Failed to create directory: [%s] for OutFile [%s]: [%s]",
        $sBaseDir, $sOutFileName, $! ) ;

        return 0 ;
      }
    }

    if ( $bCshSet ) {
      $sCmdToRun .= " >& $sOutFileName";
    }
    else {
      $sCmdToRun .= " > $sOutFileName 2>&1";
    }
  }

  $sDispCmd = $sCmdToRun; # Backup..
  $sDispCmd =~ s/$sCryptString/XXXXXXXXXX/g if ( defined $sCryptString );

  LogInt::Message( LogInt::LLTrace,
                   10199,
                   "Sys", 
                   "RunWithEnvCmdWithOutputWithTimeOut", 
                   "Going to run command line [%s], as User [%s]", 
                   $sDispCmd,
                   $sUser );

  my $bBackgroundJob = 0;
  if ( $sCmdToRun =~ /&\s*$/ ) {
      ++ $bBackgroundJob;
      $sCmdToRun =~ s/&\s*$//g;
      LogInt::Message( LogInt::LLTrace,
                       10205,
                       "Sys",
                       "RunWithEnvCmdWithOutputWithTimeOut",
                       "Running command [$sDispCmd] in background");
  }

  if ( $bSourceEnv ) {
    LogInt::Message( LogInt::LLTrace,
                     10200,
                     "Sys", 
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "Sourcing the login environment of user [%s]",
                     $sUser );
    $sCommandLine = "$::gsCommandSu - $sUser $::gsCommandArgsSu '$sCmdToRun'";
  }
  else {
    #---------------------------------------------------------------------
    # If the request is to run the command within the same user context as
    # this process is already running within, then we do not have to
    # do the command from under "su":
    #---------------------------------------------------------------------
    my $sEffectiveUser = $>;
    if( $sEffectiveUser eq $iUid ) {
      $sCommandLine = "$sCmdToRun";
    }
    else {
      $sCommandLine = "$::gsCommandSu $sUser $::gsCommandArgsSu '$sCmdToRun'";
    }
  }

  #---------------------------------------
  # Set timeout flag to true for now.
  #---------------------------------------
  $$rbTimedOut = 1; 

  if ($iTimeoutSeconds < 1 ) {
    LogInt::Message( LogInt::LLTrace,
                     10201,
                     "Sys",
                     "RunWithEnvCmdWithOutputWithTimeOut",
                     "Passed in timeout is less than 1 second, defaulting it to 15 seconds." ) ;
    $iTimeoutSeconds = 15;
  }

  my ( $iChildPid, @sCmdOut ) = ();

  $sDispCmd = $sCommandLine; # Backup..
  $sDispCmd =~ s/$sCryptString/XXXXXXXXXX/g if ( defined $sCryptString );

  LogInt::Message( LogInt::LLTraceTrace,
                   10202,
                   "Sys",
                   "RunWithEnvCmdWithOutputWithTimeOut",
                   "Now going to run complete command line [%s]", $sDispCmd );

  eval {
    local $SIG{ALRM} = sub {
      LogInt::Message( LogInt::LLTrace,
                       10203,
                       "Sys",
                       "RunWithEnvCmdWithOutputWithTimeOut",
                       "Command timed out, going to attempt to kill the blocking child processes" );
      if ( defined $iChildPid && $iChildPid > 1 ) {
        Proc::KillPIDandAllChildren( $iChildPid );
      }
      else {
        LogInt::Message( LogInt::LLTrace,
                         10204,
                         "Sys",
                         "RunWithEnvCmdWithOutputWithTimeOut",
                         "The Child PID was either undefined or less than 1.  Therefore we cannot kill the blocking child process" );
      }
      die "SIGALARM JUMP EVAL";
    };
    alarm $iTimeoutSeconds;

    my $sFullCmd = "$sCommandLine 2>&1";  
    if ( $bBackgroundJob )
    {
      $iChildPid = open(CMD, "| $sFullCmd &") 
           or LogInt::Message( LogInt::LLError,
                               10206,
                               "Sys",
                               "RunWithEnvCmdWithOutputWithTimeOut",
                               "Can't fork a process: [%s]", $! ) ; 
    }
    else
    {
      $iChildPid = open(CMD, "$sFullCmd | ") 
           or LogInt::Message( LogInt::LLError,
                               10207,
                               "Sys",
                               "RunWithEnvCmdWithOutputWithTimeOut",
                               "Can't fork a process: [%s]", $! ) ; 
    }

    if( defined $iChildPid && $iChildPid > 1 ) {
      @sCmdOut = <CMD>; 
    }
    else {
      LogInt::Message( LogInt::LLError,
                       10208,
                       "Sys",
                       "RunWithEnvCmdWithOutputWithTimeOut",
                       "The attempt to open a pipe to command [%s] resulted in an invalid child PID of [%s]",
                       $sDispCmd,
                       $iChildPid );
    }

    my $sLastOS_Error = $!;
    if( close(CMD) ) {
      $$riCommandReturnCode = $? >> 8;
    }
    else {
      $$riCommandReturnCode = $? >> 8;
      #-----------------------------------------------------------------------
      # Commands that exit non-zero, which sometimes does not indicate that
      # it failed, causes the close to return zero instead of >= 1.
      #-----------------------------------------------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10209,
                       "Sys",
                       "RunWithEnvCmdWithOutputWithTimeOut",
                       "Command line [%s] provided a non-zero exit code -- this does not necessarily indicate a problem... (Perl's OS error variable prior to the command-pipe close was [%s], and after the close was [%s] )",
                       $sDispCmd, $sLastOS_Error, $! );
    }

    @$rlsCommandOut = @sCmdOut;

    #---------------------
    # cancel the alarm
    #---------------------
    alarm 0; 

    #---------------------------
    # command didn't timeout.
    #---------------------------
    $$rbTimedOut = 0; 
  };

  if ( $$rbTimedOut ) { 
    LogInt::Message( LogInt::LLError, 
                     10210,
                     "Sys", 
                     "RunWithEnvCmdWithOutputWithTimeOut", 
                     "The command [%s] did not complete within [%s] seconds", 
                     $sDispCmd, $iTimeoutSeconds );

    return 0; # return invalid as the script timed out.
  }
  else {
    if(defined $sCryptString){
       foreach (@$rlsCommandOut){
          $_ =~ s/\b$sCryptString\b/XXXXXXXXX/;
       }
    }
    VCSagentFW::messageEngineLog( \@$rlsCommandOut, LogInt::LLTrace);
 
    return 1;
  }
}

=pod

 -------------------------------------------------------------------------
 Function :    bool UnlinkFilesAsUser ( IN List Files );

 Description : Removes files as the owner of the directory containing
               them.

 Parameters :

 @lsFiles      List of files to remove

 Returns :     boolean value in the set of {0,1} where

               0 false
               1 true
 -------------------------------------------------------------------------

=cut

sub UnlinkFilesAsUser {
  #----------------
  # Get Arguments..
  #----------------
  my ( @lsFiles ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  LogInt::Message ( LogInt::LLTrace, 10211, "Sys", "UnlinkFilesAsUser",
  "Subroutine <%s> called with:
   Files       [%s]",
  $sMyName, join ( ' ', @lsFiles ) );
 
  #-------------------------------------------
  # Set return code optimistically to online..
  #-------------------------------------------
  my $iReturnCode = 1; # Success..

  #------------------------
  # Pick only valid files..
  #------------------------
  @lsFiles = grep { -f $_ } @_;

  #-------------------
  # Return, if empty..
  #-------------------
  unless ( scalar @lsFiles ) {
    LogInt::Message ( LogInt::LLError, 10212, "Sys", "UnlinkFilesAsUser",
    "<%s> Called with empty list of files to remove", $sMyName );
    return 0;
  }

  #-------------------
  # Define variables..
  #-------------------
  my ( $sFile, $sDir, $sOwner, $iUid, $iPid, $iRetCode,
       $iWaitStatus, %hsFiles ) = ();
  
  #-------------------------
  # Stack files as per UID..
  #-------------------------
  foreach $sFile ( @lsFiles ) {
    #----------------------------
    # Get dirname for each file..
    #----------------------------
    ( $sDir = $sFile ) =~ s#(.*)/.*$#$1#; # Get dirname..

    if ( defined ( $iUid = stat ( $sDir )->uid ) ) {
      $sOwner = getpwuid ( $iUid );

      LogInt::Message ( LogInt::LLTrace, 10213, "Sys", "UnlinkFilesAsUser",
      "<%s> File [%s] Dirname [%s] UID [%s] Owner [%s]",
      $sMyName, $sFile, $sDir, $iUid, $sOwner );

      push @{ $hsFiles { $iUid } }, $sFile; # Store..
    }
  }

  $| = 1; # Auto-flush..

  #---------------------------
  # Delete files as per UIDs..
  #---------------------------
  foreach $iUid ( keys %hsFiles ) {
    #---------------------- 
    # For non-root owners..
    #---------------------- 
    if ( $iUid > 0 ) {
      #-------------------
      # Fork off a child..
      #-------------------
      if ( ( $iPid = fork () ) > 0 ) { # Parent..
        LogInt::Message ( LogInt::LLTrace, 10214, "Sys", "UnlinkFilesAsUser",
        "<%s> Running as Parent, UID [%s]. Child PID [%s]",
        $sMyName, $<, $iPid );

        #-------------
        # Reap child..
        #-------------
        if ( -1 == ( $iWaitStatus = waitpid ( $iPid, 0 ) ) ) {
          LogInt::Message ( LogInt::LLWarn, 10215, "Sys", "UnlinkFilesAsUser",
          "<%s> Wait on child [%s] failed with [%s]",
          $sMyName, $iPid, $iWaitStatus );
          $iReturnCode = 0;
        }
        else {
          #------------------------------
          # Collect exit code of child..
          #------------------------------
          $iRetCode = $? >> 8;

          LogInt::Message ( LogInt::LLTrace, 10216, "Sys", "UnlinkFilesAsUser",
          "<%s> Reaped child with PID [%s] successfully", $sMyName, $iWaitStatus );
  
          #-----------------------------
          # Collect exit code of child..
          #-----------------------------
          if ( $iRetCode ) {
            LogInt::Message ( LogInt::LLWarn, 10217, "Sys", "UnlinkFilesAsUser",
            "<%s> Child with PID [%s] exited with code [%s]",
            $sMyName, $iWaitStatus, $iRetCode );
  
            $iReturnCode = 0;
          }
          else {
            LogInt::Message ( LogInt::LLTrace, 10218, "Sys", "UnlinkFilesAsUser",
            "<%s> Child with PID [%s] exited with exit code [%s]",
            $sMyName, $iWaitStatus, $iRetCode );
          }
        }
      }
      elsif ( 0 == $iPid ) { # Child..
        LogInt::Message ( LogInt::LLTrace, 10219, "Sys", "UnlinkFilesAsUser",
        "<%s> Running as child UID [%s]. Will setuid to [%s]",
        $sMyName, $<, $iUid );
  
        #----------------------------
        # Unlink files as fileowner..
        #----------------------------
        if ( setuid ( $iUid ) ) {
          exit ( unlink ( @{ $hsFiles { $iUid } } ) ? 0 : 1 ); # Remove files..
        }
        else {
          LogInt::Message ( LogInt::LLError, 10220, "Sys", "UnlinkFilesAsUser",
          "<%s> Failed to setuid to [%s]. [%s]", $sMyName, $iUid, $! );

          exit 1; # Error..
        }
      }
      else { # Fork error..
        LogInt::Message ( LogInt::LLError, 10221, "Sys", "UnlinkFilesAsUser",
        "<%s> Could not fork [%s]", $sMyName, $! );
        $iReturnCode = 0; # Failure..
      }
    }
    else { # Owner is root..
      LogInt::Message ( LogInt::LLTrace, 10222, "Sys", "UnlinkFilesAsUser",
      "<%s> Unlinking files [%s] as root",
      $sMyName, join ( ' ', @{ $hsFiles { $iUid } } ) );

      $iReturnCode = unlink ( @{ $hsFiles { $iUid } } ); # Remove..
    }
  }

  return $iReturnCode;
}

sub RunWithEnvCmdInBG
{
  #--------------
  # Get Arguments
  #--------------
  my ( $sEnvFile,
       $bSourceEnv,
       $sCommand,
       $sCommandArgs,
       $riCommandExitCode,
       $sUser,
       $sCommandOutFile,
       $rbTimedOut,
       $iTimeoutSeconds,
       $sRunCmdFrom       ) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  #------------------
  # Announce Entry...
  #------------------
  my $sMySubName = ( caller 0 )[3];
  LogInt::Message ( LogInt::LLTrace,
  10223,
  "Sys",
  "RunWithEnvCmdInBG",
  "Subroutine <%s> called with:
  EnvFile               [%s]
  SourceEnv             [%s]
  Command               [%s]
  CommandArgs           [%s]
  User                  [%s]
  TimeoutSeconds        [%s]
  RunCmdFrom            [%s]",
  $sMyName, $sEnvFile, $bSourceEnv, $sCommand,
  $sCommandArgs, $sUser, $iTimeoutSeconds, $sRunCmdFrom );

  #--------------------------
  # Establish local variables
  #--------------------------
  my $sCmdToRun    = "";
  my $sCommandLine = "";
  my $sRedirection = "";

  if( ! defined $sUser || $sUser eq "" )
  {
    $sUser = "root";
  }

  my ( $sName,
       $sPassword,
       $iUid,
       $iGid,
       $iQuota,
       $sComment,
       $sGcos,
       $sHome,
       $sUserShell,
       $sExpire     ) = getpwnam( $sUser );

  #----------------------------------------------------------------
  # Run command from the directory specified.  Note that we do this 
  # before sourcing the EnvFile, just in case the EnvFile contains 
  # relative path name references.
  #----------------------------------------------------------------
  if( defined $sRunCmdFrom && $sRunCmdFrom ne "" )
  {
    if( -d $sRunCmdFrom )
    {
      LogInt::Message( LogInt::LLTrace, 10224, "Sys","RunWithEnvCmdInBG",
                       "Running command from directory [%s]", $sRunCmdFrom );
      $sCmdToRun .= "cd $sRunCmdFrom ;";
    }
    else
    {
      LogInt::Message( LogInt::LLError, 10225, "Sys","RunWithEnvCmdInBG", 
                       "Directory [%s] does not exist", $sRunCmdFrom );
      return 0;
    }
  }

  #-----------------------------
  # Confirm existance of EnvFile
  #-----------------------------
  $sRedirection = "> $sCommandOutFile 2>&1 &";
  if( $sEnvFile eq "" )
  {
    LogInt::Message( LogInt::LLTrace, 10226, "Sys","RunWithEnvCmdInBG",
    "Environment file not set" );
  }
  elsif( ! -r $sEnvFile )
  {
    LogInt::Message( LogInt::LLError, 10227, "Sys","RunWithEnvCmdInBG",
    "Environment file [%s] does not exist", $sEnvFile );
    return 0;
  }
  else
  {
    if( $sUserShell =~ /(.*)\/bin\/.*csh$/ )
    {
      LogInt::Message( LogInt::LLTrace, 10228, "Sys","RunWithEnvCmdInBG",
                       "User [%s] has csh derived native shell [%s]", $sUser, $sUserShell );
      #---------------------------------------------------------------
      # Shell is a csh varient, so we should use the 'source' command.
      #---------------------------------------------------------------
      $sCmdToRun .= "setenv USER $sUser; setenv HOME $sHome; source $sEnvFile;";
      $sRedirection = "& > $sCommandOutFile 2>&1";
    }
    else
    {
      LogInt::Message( LogInt::LLTrace, 10229, "Sys","RunWithEnvCmdInBG",
                       "User [%s] has sh derived native shell [%s]", $sUser, $sUserShell );
      #---------------------------------------------------------
      # Shell is one of the non-C type shells. Use '.' to source 
      # in the env file.
      #---------------------------------------------------------
      $sCmdToRun .= "USER=$sUser; HOME=$sHome; export USER HOME; . $sEnvFile ;";
      $sRedirection = "> $sCommandOutFile 2>&1 &";
    }
  }

  #--------------------------------
  # Confirm existance of Exectuable
  #--------------------------------
  if( ! -f $sCommand )
  {
    LogInt::Message( LogInt::LLError, 10230, "Sys","RunWithEnvCmdInBG",
                     "File [%s] does not exist or is not a file", $sCommand ) ;
    return 0;
  }
  elsif( ! -x $sCommand )
  {
    LogInt::Message( LogInt::LLError, 10231, "Sys","RunWithEnvCmdInBG",
                     "File [%s] is not executable", $sCommand ) ;
    return 0;
  }
  else
  {
    $sCmdToRun .= "$sCommand" . " " . "$sCommandArgs" ;
  }

  LogInt::Message( LogInt::LLTrace, 10232, "Sys","RunWithEnvCmdInBG",
                   "Going to run command line [%s], as user [%s]", $sCmdToRun, $sUser );

  if( $bSourceEnv )
  {
    LogInt::Message( LogInt::LLTrace, 10233, "Sys","RunWithEnvCmdInBG",
                     "Sourcing the login environment of user [%s]", $sUser );
    $sCommandLine = "$::gsCommandSu - $sUser $::gsCommandArgsSu '$sCmdToRun $sRedirection'";
  }
  else
  {
    #---------------------------------------------------------------------
    # If the request is to run the command within the same user context as 
    # this process is already running within, then we do not have to 
    # do the command from under "su":
    #---------------------------------------------------------------------
    my $sEffectiveUser = $> ;
    if( $sEffectiveUser eq $iUid )
    {
      $sCommandLine = "$sCmdToRun $sRedirection";
    }
    else
    {
      $sCommandLine = "$::gsCommandSu $sUser $::gsCommandArgsSu '$sCmdToRun' $sRedirection";
    }
  }

  $$rbTimedOut = 1;     # Set timeout flag to true for now.

  if( $iTimeoutSeconds < 1 )
  {
    LogInt::Message( LogInt::LLTrace, 10234, "Sys","RunWithEnvCmdInBG",
                     "Passed in timeout was less than 1 second, defaulting it to 15 seconds" );
    $iTimeoutSeconds = 15;
  }
  my $iChildPid = '';
  my @sCmdOut = ();

  LogInt::Message( LogInt::LLTrace, 10235, "Sys","RunWithEnvCmdInBG",
                   "Now going to run complete command line: [%s]", $sCommandLine );
  eval
  {
    local $SIG{ALRM} = sub
    {
      LogInt::Message( LogInt::LLTrace, 10236, "Sys","RunWithEnvCmdInBG",
                       "Command timed out, going to attempt to kill the blocking child processes" );
      if( defined $iChildPid && $iChildPid > 1 )
      {
        KillPIDandAllChildren( $iChildPid );
      }
      else
      {
        LogInt::Message( LogInt::LLError, 10237, "Sys","RunWithEnvCmdInBG",
                         "The Child PID was either undefined or less than 1.  Therefore we cannot kill the blocking child process" );
      }
      die "SIGALARM JUMP EVAL";
    };
    alarm $iTimeoutSeconds;

    #my $sFullCmd = "$sCommandLine 2>&1";  
    my $sFullCmd = "$sCommandLine";  
    LogInt::Message( LogInt::LLTrace, 10238, "Sys","RunWithEnvCmdInBG",
                     "FullCmd is [%s]",$sFullCmd);
    $iChildPid = open( CMD, "$sFullCmd |" ) ||
    &logError( 10239, "Can't fork a process" ); 
    if( defined $iChildPid && $iChildPid > 1 ) 
    {
      while( <CMD> )
      { 
        push( @sCmdOut, $_ );
      }
    }
    else
    {
      LogInt::Message( LogInt::LLError, 10240, "Sys","RunWithEnvCmdInBG",
                       "The attempt to open a pipe to command [%s] resulted in an invalid child PID of [%s]",
                          $sFullCmd,
                          $iChildPid );
    }
    my $sLastOS_Error = $!;
    if( ! close( CMD ) )
    {
      # Commands that exit non-zero, which sometimes does not indicate that 
      # it failed, causes the close to return zero instead of >= 1.  
      LogInt::Message( LogInt::LLTrace, 10241, "Sys","RunWithEnvCmdInBG",
      "Command line [%s] provided a non-zero exit code -- this does not necessarily indicate a problem... (Perl's OS error variable prior to the command-pipe close was [%s], and after the close was [%s] )",
                          $sFullCmd,
                          $sLastOS_Error,
                          $! );
    }


    $$riCommandExitCode = $? >> 8;
    alarm 0; # cancel the alarm
    $$rbTimedOut = 0; #command didn't timeout.
  };

  if( $$rbTimedOut )
  { 
    LogInt::Message( LogInt::LLError, 10242, "Sys","RunWithEnvCmdInBG",
    "The command [%s] did not complete within [%s] seconds", 
                        $sCommandLine,
                        $iTimeoutSeconds );
    return 0;
  }
  else
  {
    LogInt::Message( LogInt::LLTrace, 10243, "Sys","RunWithEnvCmdInBG",
                     "Command [%s] with args [%s] returned code [%s]",
                        $sCommand,
                        $sCommandArgs,
                        $$riCommandExitCode );
    return 1;
  }
}

1; 
